@@ -24,68 +24,126 @@
 #include <string.h>
 #include <stdlib.h>
 
+#include "globals.h"
 #include "config.h"
 #include "utils.h"
 
-#define alnum(c)	(isalpha(c) || isdigit(c))
-#define blank(c)	((c) == ' ' || (c) == '\t' || (c) == '\r' || (c) == '\n')
+/*
+static const char *globals[] = {
+	"Allow",
+	"Deny",
+	"Gateway",
+	"Listen",
+	"SOCKS5Proxy",
+	"SOCKS5User",
+	"NTLMToBasic",
+	"Tunnel" };
+*/
 
 config_t config_open(const char *fname) {
-	config_t ret;
+	config_t rc;
 	FILE *fp;
-	char *buf, *key, *value;
-	int i, j, len;
+	char *buf, *tmp, *key, *value;
+	char section[MINIBUF_SIZE] = "global";
+	int i, j, slen, len, quote;
+
+	//printf("sizeof = %d\n", sizeof(globals) / sizeof(char *));
 
 	fp = fopen(fname, "r");
 	if (!fp)
 		return NULL;
 	
 	buf = new(BUFSIZE);
-	ret = (config_t)new(sizeof(struct config_s));
-	ret->options = NULL;
+	rc = (config_t)new(sizeof(struct config_s));
+	rc->options = NULL;
 
 	while (!feof(fp)) {
-		fgets(buf, BUFSIZE, fp);
-		len = MIN(BUFSIZE, strlen(buf));
+		quote = 0;
+		tmp = fgets(buf, BUFSIZE, fp);
+		if (!tmp)
+			break;
 
+		len = MIN(BUFSIZE, strlen(buf));
 		if (!len || feof(fp))
 			continue;
 
-		i = j = 0;
-		while (j < len && blank(buf[j]))
-			j++;
+		/*
+		 * Find first non-empty character
+		 */
+		for (i = j = 0; j < len && isspace(buf[j]); ++j);
 
-		if (j >= len || buf[j] == '#')
+		/*
+		 * Comment?
+		 */
+		if (j >= len || buf[j] == '#' || buf[j] == ';')
 			continue;
 
-		i = j;
-		while (j < len && alnum(buf[j]))
-			j++;
+		/*
+		 * Find end of keyword
+		 */
+		for (i = j; j < len && isalnum(buf[j]); ++j);
 
-		if (j >= len || !blank(buf[j]))
+		/*
+		 * Malformed?
+		 */
+		if (j >= len)
 			continue;
 
+		/*
+		 * Is it a section?
+		 */
+		if (buf[j] == '[') {
+			for (++j; j < len && isspace(buf[j]); ++j);
+			for (slen = j; j < len && j-slen < MINIBUF_SIZE-1 && buf[j] != ']' && !isspace(buf[j]); ++j);
+			if (j-slen > 0) {
+				strlcpy(section, buf+slen, j-slen+1);
+			}
+			continue;
+		}
+
+		/*
+		 * It's an OK keyword
+		 */
 		key = substr(buf, i, j-i);
-		i = j;
-		while (j < len && blank(buf[j]))
-			j++;
 
-		if (j >= len || buf[j] == '#')
+		/*
+		 * Find next non-empty character
+		 */
+		for (i = j; j < len && isspace(buf[j]); ++j);
+		if (j >= len || buf[j] == '#' || buf[j] == ';')
 			continue;
 
-		value = substr(buf, j, len-j);
-		i = strcspn(value, "#");
-		if (i != strlen(value))
-			value[i] = 0;
+		/*
+		 * Is value quoted?
+		 */
+		if (buf[j] == '"') {
+			quote = 1;
+			for (i = ++j; j < len && buf[i] != '"'; ++i);
+			if (i >= len)
+				continue;
+		} else
+			i = len;
 
-		trimr(value);
-		ret->options = hlist_add(ret->options, key, value, 0, 0);
+		/*
+		 * Get value as quoted or until EOL/comment
+		 */
+		value = substr(buf, j, i-j);
+		if (!quote) {
+			i = strcspn(value, "#");
+			if (i != strlen(value))
+				value[i] = 0;
+			trimr(value);
+		}
+
+		if (debug)
+			printf("section: %s, %s = '%s'\n", section, key, value);
+		rc->options = hlist_add(rc->options, key, value, HLIST_NOALLOC, HLIST_NOALLOC);
 	}
 
 	free(buf);
 	fclose(fp);
 
-	return ret;
+	return rc;
 }
 
 void config_set(config_t cf, char *option, char *value) {
