@@ -22,14 +22,34 @@
 #ifndef _UTILS_H
 #define _UTILS_H
 
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
+# include <sys/param.h>
+#endif
 #include <pthread.h>
+#include <netinet/in.h>
+
 #include "config/config.h"
 
-#define BUFSIZE			1024
+#define BUFSIZE			4096
 #define MINIBUF_SIZE		50
 #define VAL(var, type, offset)	*((type *)(var+offset))
 #define MEM(var, type, offset)	(type *)(var+offset)
-#define MIN(a, b)		((a) < (b) ? (a) : (b))
+
+#if !defined(__FreeBSD__) && !defined(__NetBSD__) && !defined(__OpenBSD__)
+# define MIN(a, b)		((a) < (b) ? (a) : (b))
+#endif
+
+/*
+#define isalnum(c)	(isalpha(c) || isdigit(c))
+#define isspace(c)	((c) == ' ' || (c) == '\f' || (c) == '\t' || (c) == '\r' || (c) == '\n')
+*/
+
+/*
+ * Solaris doesn't have LOG_PERROR
+ */
+#ifndef LOG_PERROR
+# define LOG_PERROR	LOG_CONS
+#endif
 
 /*
  * Two single-linked list types. First is for storing headers,
@@ -40,16 +60,22 @@ typedef struct hlist_s *hlist_t;
 struct hlist_s {
 	char *key;
 	char *value;
+	int islist;
 	struct hlist_s *next;
 };
 
 typedef struct plist_s *plist_t;
 struct plist_s {
 	unsigned long key;
-	char *aux;
+	void *aux;
 	struct plist_s *next;
 };
 
+typedef enum {
+	HLIST_NOALLOC = 0,
+	HLIST_ALLOC
+} hlist_add_t;
+
 /*
  * Request/response data structure. Complete and parsed req/res is
  * kept in this. See below for (de)allocation routines.
@@ -60,10 +86,17 @@ struct rr_data_s {
 	hlist_t headers;
 	int code;
 	int skip_http;
+	int body_len;
+	int empty;
+	int port;
 	char *method;
 	char *url;
+	char *rel_url;
+	char *hostname;
 	char *http;
 	char *msg;
+	char *body;
+	char *errmsg;
 };
 
 /*
@@ -72,18 +105,22 @@ struct rr_data_s {
 struct thread_arg_s {
 	int fd;
 	char *target;
+	struct sockaddr_in addr;
 };
 
-extern plist_t plist_add(plist_t list, unsigned long key, char *aux);
+extern void myexit(int rc);
+extern void croak(const char *msg, int console);
+
+extern plist_t plist_add(plist_t list, unsigned long key, void *aux);
 extern plist_t plist_del(plist_t list, unsigned long key);
 extern int plist_in(plist_t list, unsigned long key);
 extern void plist_dump(plist_t list);
 extern char *plist_get(plist_t list, int key);
-extern int plist_pop(plist_t *list);
+extern int plist_pop(plist_t *list, void **aux);
 extern int plist_count(plist_t list);
 extern plist_t plist_free(plist_t list);
 
-extern hlist_t hlist_add(hlist_t list, char *key, char *value, int allockey, int allocvalue);
+extern hlist_t hlist_add(hlist_t list, char *key, char *value, hlist_add_t allockey, hlist_add_t allocvalue);
 extern hlist_t hlist_dup(hlist_t list);
 extern hlist_t hlist_del(hlist_t list, const char *key);
 extern hlist_t hlist_mod(hlist_t list, char *key, char *value, int add);
@@ -91,6 +128,7 @@ extern int hlist_in(hlist_t list, const char *key);
 extern int hlist_count(hlist_t list);
 extern char *hlist_get(hlist_t list, const char *key);
 extern int hlist_subcmp(hlist_t list, const char *key, const char *substr);
+extern int hlist_subcmp_all(hlist_t list, const char *key, const char *substr);
 extern hlist_t hlist_free(hlist_t list);
 extern void hlist_dump(hlist_t list);
 
@@ -100,15 +138,14 @@ extern size_t strlcat(char *dst, const char *src, size_t siz);
 extern char *trimr(char *buf);
 extern char *lowercase(char *str);
 extern char *uppercase(char *str);
-extern int head_ok(const char *src);
-extern char *head_name(const char *src);
-extern char *head_value(const char *src);
 extern int unicode(char **dst, char *src);
 extern char *new(size_t size);
 extern char *urlencode(const char *str);
 
 extern rr_data_t new_rr_data(void);
+extern rr_data_t copy_rr_data(rr_data_t dst, rr_data_t src);
 extern rr_data_t dup_rr_data(rr_data_t data);
+extern rr_data_t reset_rr_data(rr_data_t data);
 extern void free_rr_data(rr_data_t data);
 
 extern char *printmem(char *src, size_t len, int bitwidth);
@@ -121,6 +158,8 @@ extern long int random(void);
 #if config_gethostname == 1
 extern int gethostname(char *name, size_t len);
 #endif
+#ifndef strdup
 extern char *strdup(const char *src);
+#endif
 
 #endif /* _UTILS_H */
